@@ -1,3 +1,4 @@
+# frozen_string_literal: false
 require 'test/unit'
 
 class TestSprintf < Test::Unit::TestCase
@@ -83,18 +84,6 @@ class TestSprintf < Test::Unit::TestCase
     assert_equal("NaN", sprintf("%-f", nan))
     assert_equal("+NaN", sprintf("%+f", nan))
 
-    assert_equal("NaN", sprintf("%3f", nan))
-    assert_equal("NaN", sprintf("%-3f", nan))
-    assert_equal("+NaN", sprintf("%+3f", nan))
-
-    assert_equal(" NaN", sprintf("% 3f", nan))
-    assert_equal(" NaN", sprintf("%- 3f", nan))
-    assert_equal("+NaN", sprintf("%+ 3f", nan))
-
-    assert_equal(" NaN", sprintf("% 03f", nan))
-    assert_equal(" NaN", sprintf("%- 03f", nan))
-    assert_equal("+NaN", sprintf("%+ 03f", nan))
-
     assert_equal("     NaN", sprintf("%8f", nan))
     assert_equal("NaN     ", sprintf("%-8f", nan))
     assert_equal("    +NaN", sprintf("%+8f", nan))
@@ -118,26 +107,6 @@ class TestSprintf < Test::Unit::TestCase
     assert_equal("Inf", sprintf("%-f", inf))
     assert_equal("+Inf", sprintf("%+f", inf))
 
-    assert_equal(" Inf", sprintf("% f", inf))
-    assert_equal(" Inf", sprintf("%- f", inf))
-    assert_equal("+Inf", sprintf("%+ f", inf))
-
-    assert_equal(" Inf", sprintf("% 0f", inf))
-    assert_equal(" Inf", sprintf("%- 0f", inf))
-    assert_equal("+Inf", sprintf("%+ 0f", inf))
-
-    assert_equal("Inf", sprintf("%3f", inf))
-    assert_equal("Inf", sprintf("%-3f", inf))
-    assert_equal("+Inf", sprintf("%+3f", inf))
-
-    assert_equal(" Inf", sprintf("% 3f", inf))
-    assert_equal(" Inf", sprintf("%- 3f", inf))
-    assert_equal("+Inf", sprintf("%+ 3f", inf))
-
-    assert_equal(" Inf", sprintf("% 03f", inf))
-    assert_equal(" Inf", sprintf("%- 03f", inf))
-    assert_equal("+Inf", sprintf("%+ 03f", inf))
-
     assert_equal("     Inf", sprintf("%8f", inf))
     assert_equal("Inf     ", sprintf("%-8f", inf))
     assert_equal("    +Inf", sprintf("%+8f", inf))
@@ -158,26 +127,6 @@ class TestSprintf < Test::Unit::TestCase
     assert_equal("-Inf", sprintf("%-f", -inf))
     assert_equal("-Inf", sprintf("%+f", -inf))
 
-    assert_equal("-Inf", sprintf("% f", -inf))
-    assert_equal("-Inf", sprintf("%- f", -inf))
-    assert_equal("-Inf", sprintf("%+ f", -inf))
-
-    assert_equal("-Inf", sprintf("% 0f", -inf))
-    assert_equal("-Inf", sprintf("%- 0f", -inf))
-    assert_equal("-Inf", sprintf("%+ 0f", -inf))
-
-    assert_equal("-Inf", sprintf("%4f", -inf))
-    assert_equal("-Inf", sprintf("%-4f", -inf))
-    assert_equal("-Inf", sprintf("%+4f", -inf))
-
-    assert_equal("-Inf", sprintf("% 4f", -inf))
-    assert_equal("-Inf", sprintf("%- 4f", -inf))
-    assert_equal("-Inf", sprintf("%+ 4f", -inf))
-
-    assert_equal("-Inf", sprintf("% 04f", -inf))
-    assert_equal("-Inf", sprintf("%- 04f", -inf))
-    assert_equal("-Inf", sprintf("%+ 04f", -inf))
-
     assert_equal("    -Inf", sprintf("%8f", -inf))
     assert_equal("-Inf    ", sprintf("%-8f", -inf))
     assert_equal("    -Inf", sprintf("%+8f", -inf))
@@ -200,11 +149,6 @@ class TestSprintf < Test::Unit::TestCase
     assert_equal(" Inf", sprintf("% e", inf), '[ruby-dev:34002]')
   end
 
-  def test_bignum
-    assert_match(/\A10{120}\.0+\z/, sprintf("%f", 100**60))
-    assert_match(/\A10{180}\.0+\z/, sprintf("%f", 1000**60))
-  end
-
   def test_rational
     assert_match(/\A0\.10+\z/, sprintf("%.60f", 0.1r))
     assert_match(/\A0\.010+\z/, sprintf("%.60f", 0.01r))
@@ -474,8 +418,14 @@ class TestSprintf < Test::Unit::TestCase
     end
   end
 
-  def test_width_underflow
-    bug = 'https://github.com/mruby/mruby/issues/3347'
-    assert_equal("!", sprintf("%*c", 0, ?!.ord), bug)
+  def test_named_default
+    h = Hash.new('world')
+    assert_equal("hello world", "hello %{location}" % h)
+    assert_equal("hello world", "hello %<location>s" % h)
+  end
+
+  def test_named_with_nil
+    h = { key: nil, key2: "key2_val" }
+    assert_equal("key is , key2 is key2_val", "key is %{key}, key2 is %{key2}" % h)
   end
 end
